import numpy as np


def reward_function(params):
    # 입력 파라미터 추출
    track_width = params['track_width']
    distance_from_center = params['distance_from_center']
    all_wheels_on_track = params['all_wheels_on_track']
    heading = params['heading']
    waypoints = params['waypoints']
    closest_waypoint = params['closest_waypoints']
    speed = params['speed']

    # 보상 초기값
    reward = 1.0

    # 1. 트랙을 벗어나지 않도록 보상
    if not all_wheels_on_track:
        return 1e-3  # 트랙을 벗어나면 매우 작은 보상

    # 2. 헤더 방향이 중앙선과 잘 맞을수록 보상
    next_waypoint = waypoints[closest_waypoint[1]]  # 다음 웨이포인트
    prev_waypoint = waypoints[closest_waypoint[0]]  # 이전 웨이포인트
    track_direction = np.arctan2(next_waypoint[1] - prev_waypoint[1], next_waypoint[0] - prev_waypoint[0])
    track_direction = np.degrees(track_direction)  # 라디안 값을 각도로 변환

    # 헤더와 트랙 방향의 차이
    direction_diff = abs(heading - track_direction)

    # 방향 차이가 작을수록 보상 증가
    if direction_diff < 10.0:
        reward += 1.0
    elif direction_diff < 20.0:
        reward += 0.5
    else:
        reward += 0.1  # 차이가 크면 보상 적음

    # 3. 속도에 따른 보상
    # 빠르게 주행할수록 보상 증가, 하지만 안정적인 범위 내에서 보상 강화
    SPEED_THRESHOLD = 3.0  # 적절한 속도 기준 설정 (m/s)

    if speed >= SPEED_THRESHOLD:
        reward += 1.5  # 빠른 속도일 경우 보상 강화
    else:
        reward += 0.5  # 느린 속도일 경우 보상 감소

    # 4. 트랙 중앙에서 너무 멀어지면 보상 감소
    # 트랙의 중앙에서 너무 멀면 패널티
    marker_1 = 0.1 * track_width
    marker_2 = 0.25 * track_width
    marker_3 = 0.5 * track_width

    if distance_from_center <= marker_1:
        reward += 1.0  # 중앙에 가까울수록 보상
    elif distance_from_center <= marker_2:
        reward += 0.5
    elif distance_from_center <= marker_3:
        reward += 0.1
    else:
        reward = 1e-3  # 트랙에서 너무 멀면 큰 패널티

    return float(reward)
